package com.dj.service.sys.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.dj.comparator.GroupComparator;
//import com.dj.comparator.DeptComparator;
import com.dj.dao.base.BaseDaoI;
import com.dj.model.sys.Tgroup;
import com.dj.pageModel.base.TreeNode;
import com.dj.pageModel.sys.Group;
import com.dj.service.base.impl.BaseServiceImpl;
import com.dj.service.sys.GroupServiceI;


@Service("groupService")
public class GroupServiceImpl extends BaseServiceImpl implements GroupServiceI {

	private BaseDaoI<Tgroup> groupDao;

	public BaseDaoI<Tgroup> getGroupDao() {
		return groupDao;
	}

	@Autowired
	public void setGroupDao(BaseDaoI<Tgroup> groupDao) {
		this.groupDao = groupDao;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Group> findAll() {
		List<Group> ld = new ArrayList<Group>();
		String hql = "from Tgroup t order by t.cseq";
		List<Tgroup> ltd = groupDao.find(hql);
		for (Tgroup td : ltd) {
			Group d = new Group();
			d.setCid(td.getCid());
			d.setCname(td.getCname());
			d.setCiconcls(td.getCiconcls());
			if(td.getTgroup()!=null){
				d.setPid(td.getTgroup().getCid());
			}
			ld.add(d);
		}
		return ld;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<TreeNode> tree(Group group, Boolean b) {
		List<Object> param = new ArrayList<Object>();
		String hql = "from Tgroup t where t.tgroup is null order by t.cseq";
		if (group != null && group.getId() != null && !group.getId().trim().equals("")) {
			hql = "from Tgroup t where t.tgroup.cid = ? order by t.cseq";
			param.add(group.getId());
		}
		List<Tgroup> l = groupDao.find(hql, param);
		List<TreeNode> tree = new ArrayList<TreeNode>();
		for (Tgroup t : l) {
			tree.add(this.tree(t, b));
		}
		return tree;
	}

	private TreeNode tree(Tgroup t, boolean recursive) {
		TreeNode node = new TreeNode();
		node.setId(t.getCid());
		node.setText(t.getCname());
		if (t.getCiconcls() != null) {
			node.setIconCls(t.getCiconcls());
		} else {
			node.setIconCls("");
		}
		if (t.getTgroups() != null && t.getTgroups().size() > 0) {
			node.setState("closed");
			if (recursive) {// 递归查询子节点
				List<Tgroup> l = new ArrayList<Tgroup>(t.getTgroups());
				Collections.sort(l, new GroupComparator());// 排序
				List<TreeNode> children = new ArrayList<TreeNode>();
				for (Tgroup r : l) {
					TreeNode tn = tree(r, true);
					children.add(tn);
				}
				node.setChildren(children);
				node.setState("open");
			}
		}
		return node;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Group> treegrid(Group group) {
		List<Tgroup> l;
		if (group != null && group.getId() != null) {
			l = groupDao.find("from Tgroup t where t.tgroup.cid = ? order by t.cseq", new Object[] { group.getId() });
		} else {
			l = groupDao.find("from Tgroup t where t.tgroup is null order by t.cseq");
		}
		return changeModel(l);
	}

	private List<Group> changeModel(List<Tgroup> Tgroups) {
		List<Group> l = new ArrayList<Group>();
		if (Tgroups != null && Tgroups.size() > 0) {
			for (Tgroup t : Tgroups) {
				Group m = new Group();
				BeanUtils.copyProperties(t, m);
				if (t.getTgroup() != null) {
					m.setPid(t.getTgroup().getCid());
					m.setPname(t.getTgroup().getCname());
				}
				if (countChildren(t.getCid()) > 0) {
					m.setState("closed");
				}
				if (t.getCiconcls() == null) {
					m.setIconCls("");
				} else {
					m.setIconCls(t.getCiconcls());
				}
				l.add(m);
			}
		}
		return l;
	}

	/**
	 * 统计有多少子节点
	 */
	private Long countChildren(String pid) {
		return groupDao.count("select count(*) from Tgroup t where t.tgroup.cid = ?", new Object[] { pid });
	}

	public void delete(Group group) {
		del(group.getCid());
	}

	private void del(String cid) {
		Tgroup t = groupDao.get(Tgroup.class, cid);
		if (t != null) {
			Set<Tgroup> groups = t.getTgroups();
			if (groups != null && !groups.isEmpty()) {
				// 说明当前菜单下面还有子菜单
				for (Tgroup tgroup : groups) {
					del(tgroup.getCid());
				}
			}
			groupDao.delete(t);
		}
	}

	public void add(Group group) {
		Tgroup t = new Tgroup();
		BeanUtils.copyProperties(group, t);
		t.setCid(UUID.randomUUID().toString());
		if (group.getPid() != null) {
			t.setTgroup(groupDao.get(Tgroup.class, group.getPid()));
		}
		if (group.getIconCls() != null) {
			t.setCiconcls(group.getIconCls());
		}
		groupDao.save(t);
	}

	public void edit(Group group) {
		Tgroup t = groupDao.get(Tgroup.class, group.getCid());
		BeanUtils.copyProperties(group, t);
		if (group.getIconCls() != null) {
			t.setCiconcls(group.getIconCls());
		}
		if (group.getPid() != null && !group.getPid().equals(group.getCid())) {
			Tgroup pt = groupDao.get(Tgroup.class, group.getPid());
			if (pt != null) {
				if (isDown(t, pt)) {// 要将当前节点修改到当前节点的子节点中
					Set<Tgroup> tgroups = t.getTgroups();// 当前要修改的节点的所有下级节点
					if (tgroups != null && tgroups.size() > 0) {
						for (Tgroup tgroup : tgroups) {
							if (tgroup != null) {
								tgroup.setTgroup(null);
							}
						}
					}
				}
				t.setTgroup(pt);
			}

		}
	}

	/**
	 * 判断是否是将当前节点修改到当前节点的子节点
	 * 
	 * @param t
	 * @param pt
	 * @return
	 */
	private boolean isDown(Tgroup t, Tgroup pt) {
		if (pt.getTgroup() != null) {
			if (pt.getTgroup().getCid().equals(t.getCid())) {
				return true;
			} else {
				return isDown(t, pt.getTgroup());
			}
		}
		return false;
	}

}
