package com.dj.service.sys.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import com.dj.comparator.PostComparator;
import com.dj.dao.base.BaseDaoI;
import com.dj.model.sys.Tpost;
import com.dj.pageModel.base.TreeNode;
import com.dj.pageModel.sys.Post;
import com.dj.service.base.impl.BaseServiceImpl;
import com.dj.service.sys.PostServiceI;

@Service("postService")
public class PostServiceImpl extends BaseServiceImpl implements PostServiceI {

	private BaseDaoI<Tpost> postDao;

	public BaseDaoI<Tpost> getPostDao() {
		return postDao;
	}

	@Autowired
	public void setPostDao(BaseDaoI<Tpost> postDao) {
		this.postDao = postDao;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Post> findAll() {
		List<Post> ld = new ArrayList<Post>();
		String hql = "from Tpost t order by t.cseq";
		List<Tpost> ltd = postDao.find(hql);
		for (Tpost td : ltd) {
			Post d = new Post();
			d.setCid(td.getCid());
			d.setCname(td.getCname());
			d.setCiconcls(td.getCiconcls());
			if (td.getTpost() != null) {
				d.setPid(td.getTpost().getCid());
			}
			ld.add(d);
		}
		return ld;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<TreeNode> tree(Post post, Boolean b) {
		List<Object> param = new ArrayList<Object>();
		String hql = "from Tpost t where t.tpost is null order by t.cseq";
		if (post != null && post.getId() != null
				&& !post.getId().trim().equals("")) {
			hql = "from Tpost t where t.tpost.cid = ? order by t.cseq";
			param.add(post.getId());
		}
		List<Tpost> l = postDao.find(hql, param);
		List<TreeNode> tree = new ArrayList<TreeNode>();
		for (Tpost t : l) {
			tree.add(this.tree(t, b));
		}
		return tree;
	}

	private TreeNode tree(Tpost t, boolean recursive) {
		TreeNode node = new TreeNode();
		node.setId(t.getCid());
		node.setText(t.getCname());
		if (t.getCiconcls() != null) {
			node.setIconCls(t.getCiconcls());
		} else {
			node.setIconCls("");
		}
		if (t.getTposts() != null && t.getTposts().size() > 0) {
			node.setState("closed");
			if (recursive) {// 递归查询子节点
				List<Tpost> l = new ArrayList<Tpost>(t.getTposts());
				Collections.sort(l, new PostComparator());// 排序
				List<TreeNode> children = new ArrayList<TreeNode>();
				for (Tpost r : l) {
					TreeNode tn = tree(r, true);
					children.add(tn);
				}
				node.setChildren(children);
				node.setState("open");
			}
		}
		return node;
	}

	@Transactional(readOnly = true, propagation = Propagation.SUPPORTS)
	public List<Post> treegrid(Post post) {
		List<Tpost> l;
		if (post != null && post.getId() != null) {
			l = postDao.find(
					"from Tpost t where t.tpost.cid = ? order by t.cseq",
					new Object[] { post.getId() });
		} else {
			l = postDao
					.find("from Tpost t where t.tpost is null order by t.cseq");
		}
		return changeModel(l);
	}

	private List<Post> changeModel(List<Tpost> Tposts) {
		List<Post> l = new ArrayList<Post>();
		if (Tposts != null && Tposts.size() > 0) {
			for (Tpost t : Tposts) {
				Post m = new Post();
				BeanUtils.copyProperties(t, m);
				if (t.getTpost() != null) {
					m.setPid(t.getTpost().getCid());
					m.setPname(t.getTpost().getCname());
				}
				if (countChildren(t.getCid()) > 0) {
					m.setState("closed");
				}
				if (t.getCiconcls() == null) {
					m.setIconCls("");
				} else {
					m.setIconCls(t.getCiconcls());
				}
				l.add(m);
			}
		}
		return l;
	}

	/**
	 * 统计有多少子节点
	 */
	private Long countChildren(String pid) {
		return postDao.count(
				"select count(*) from Tpost t where t.tpost.cid = ?",
				new Object[] { pid });
	}

	public void delete(Post post) {
		del(post.getCid());
	}

	private void del(String cid) {
		Tpost t = postDao.get(Tpost.class, cid);
		if (t != null) {
			Set<Tpost> posts = t.getTposts();
			if (posts != null && !posts.isEmpty()) {
				// 说明当前菜单下面还有子菜单
				for (Tpost tpost : posts) {
					del(tpost.getCid());
				}
			}
			postDao.delete(t);
		}
	}

	public void add(Post post) {
		Tpost t = new Tpost();
		BeanUtils.copyProperties(post, t);
		t.setCid(UUID.randomUUID().toString());
		if (post.getPid() != null) {
			t.setTpost(postDao.get(Tpost.class, post.getPid()));
		}
		if (post.getIconCls() != null) {
			t.setCiconcls(post.getIconCls());
		}
		postDao.save(t);
	}

	public void edit(Post post) {
		Tpost t = postDao.get(Tpost.class, post.getCid());
		BeanUtils.copyProperties(post, t);
		if (post.getIconCls() != null) {
			t.setCiconcls(post.getIconCls());
		}
		if (post.getPid() != null && !post.getPid().equals(post.getCid())) {
			Tpost pt = postDao.get(Tpost.class, post.getPid());
			if (pt != null) {
				if (isDown(t, pt)) {// 要将当前节点修改到当前节点的子节点中
					Set<Tpost> tposts = t.getTposts();// 当前要修改的节点的所有下级节点
					if (tposts != null && tposts.size() > 0) {
						for (Tpost tpost : tposts) {
							if (tpost != null) {
								tpost.setTpost(null);
							}
						}
					}
				}
				t.setTpost(pt);
			}

		}
	}

	/**
	 * 判断是否是将当前节点修改到当前节点的子节点
	 * 
	 * @param t
	 * @param pt
	 * @return
	 */
	private boolean isDown(Tpost t, Tpost pt) {
		if (pt.getTpost() != null) {
			if (pt.getTpost().getCid().equals(t.getCid())) {
				return true;
			} else {
				return isDown(t, pt.getTpost());
			}
		}
		return false;
	}

}
